

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Scanner;
import java.util.Set;

public class JavaApp {


    public static void main(String[] args) throws IOException {
//        Scanner sc = new Scanner(System.in);
        BufferedReader sc = new BufferedReader(new InputStreamReader(System.in));
        
        String[] line = sc.readLine().split("[ ]");
        int nodeCount = Integer.parseInt(line[0]);
        int eventCount = Integer.parseInt(line[1]);
        
        Island[] islands = new Island[nodeCount];
        for (int i = 0; i < islands.length; i++) {
            islands[i] = new Island(i);
        }
        
        for (int i = 0; i < eventCount; i++) {
            String[] ln = sc.readLine().split("[ ]");
            int eventType = Integer.parseInt(line[0]);
            
            if (eventType == 0) {
                int x = Integer.parseInt(ln[1]);
                int y = Integer.parseInt(ln[2]);
                int v = Integer.parseInt(ln[3]);
                
                Island i1 = islands[x];
                Island i2 = islands[y];
                Bridge newBridge = new Bridge(v, i1, i2);
                
                i1.edges.put(y, newBridge);
                i2.edges.put(x, newBridge);
                
            } else if (eventType == 1) {
                
                int x = Integer.parseInt(ln[1]);
                int y = Integer.parseInt(ln[2]);
                
                Island i1 = islands[x];
                Island i2 = islands[y];
                
                i1.edges.remove(y);
                i2.edges.remove(x);
                
            } else if (eventType == 2) {
                int x = Integer.parseInt(ln[1]);
                int y = Integer.parseInt(ln[2]);
                
                Island i1 = islands[x];
                Island i2 = islands[y];
                
                findPath(islands, i1, i2);
                
                if (i2.x == null) {
                    System.out.println("-1");
                } else {
                    System.out.println(mostDangerous(i2) + "");
                }
            }
        }
        
    }
    
    private static int mostDangerous(Island end) {
        Island node = end;
        int mostDanger = Integer.MIN_VALUE;
        while (node.x != null) {
            int dng = node.edges.get(node.x.ID).dagner;
            if (dng < mostDanger) {
                mostDanger = dng;
            }
        }
        
        return mostDanger;
    }
    
    private static void findPath(Island[] nodes, Island start, Island end) {
        init(nodes);
        
        start.t = 0;
        PriorityQueue<Island> queue = new PriorityQueue<Island>();
        queue.add(start);
        
        while (!queue.isEmpty()) {
            Island poped = queue.poll();
            
            if (poped == end) {
                break;
            }
            
            for (Bridge edge : poped.edges.values()) {
                Island target = edge.getTarget(poped);
                int newT = poped.t + edge.dagner;
                
                if (newT < target.t) {
                    target.t = newT;
                    target.x = poped;
                    queue.add(target);
                }
            }
        }
        
    }
    
    private static void init(Island[] nodes) {
        for (Island node : nodes) {
            node.t = Integer.MAX_VALUE;
            node.x = null;
                    
        }
    }
    
    static class Island implements Comparable<Island> {
        
        int ID;
        int t;
        Island x;        
        Map<Integer, Bridge> edges;

        public Island(int id) {
            this.ID = id;
            this.t = Integer.MAX_VALUE;
            this.x = null;
            this.edges = new HashMap<>();
        }

        @Override
        public int compareTo(Island o) {
            return this.t < o.t ? -1 : 1;
        }
        
        
        
    }
    
    static class Bridge {
        
        int dagner;
        Island i1;
        Island i2;
        
        Bridge(int dagnge, Island i1, Island i2) {
            this.dagner = dagnge;
            this.i1 = i1;
            this.i2 = i2;
        }
        
        Island getTarget(Island src) {
            return src == this.i1 ? this.i2 : this.i1;
        }
        
    }
}
